/*
	$Id $

	Copyright (C) 2017 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.SessionControl.UnverifiedDeviceInfo;
import net.sf.basedb.core.authentication.LoginRequest;

/**
	This exception can be thrown when a user is trying to login
	from a device that has not been verified. A verification code
	has also been sent to the user (by email). To get more information
	about the device use {@link SessionControl#getUnverifiedDeviceInfo()}.
	<p>
	
	The client application should present a form where the user can enter 
	the verification code and then call 
	{@link SessionControl#verifyDevice(String, boolean)}.
	<p>
	
	When the device has been verified correctly the client application
	should store the token from {@link UnverifiedDeviceInfo#getDeviceToken()}.
	The next time the user is logging in, the stored token should be submitted 
	together with other login information. See {@link LoginRequest#setDeviceToken(String)}.

	@author Nicklas
	@since 3.12
*/
public class DeviceNotVerifiedException
	extends BaseException
{

	private static final long serialVersionUID = -6864074877391853773L;


	public DeviceNotVerifiedException()
	{}
	
}

