/**
	$Id $

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
	Annotation (java) that can be used by {@link Subtypable} items to 
	enable/disable support for pushing annotations (BASE) to parent
	items. See {@link ItemSubtype#getPushAnnotations()}.
	The annotation should be put on the 
	{@link Subtypable#getItemSubtype()} method. Example:
	
	<pre class="code">
&#064;Override
&#064;SupportsPushAnnotations
public ItemSubtype getItemSubtype()
{
...
}
</pre>

	@author Nicklas
	@since 3.1
	@base.modified $Date$
*/
@Documented
@Target(ElementType.METHOD)
@Retention(RetentionPolicy.RUNTIME)
public @interface SupportsPushAnnotations
{}