/**
	$Id: BioPlateEventTypeData.java 7015 2015-11-17 12:57:57Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	Each bioplate event must have a type that defines what happened. The actual
	implementation for each type needs to be done in the gui and it is usually
	not possible to convert an event to a different type.

	@author Nicklas
	@since 2.17
 	@see net.sf.basedb.core.BioPlateEventType
	@see <a href="../../../../../../html/developer/api/data_api.biomaterials.html">Developer documentation: Biomaterials LIMS</a>
	@base.modified $Date: 2015-11-17 13:57:57 +0100 (ti, 17 nov 2015) $
	@hibernate.class table="`BioPlateEventTypes`" lazy="false"
*/
public class BioPlateEventTypeData
	extends BasicData
	implements NameableData, RemovableData, SystemData
{
	
	public BioPlateEventTypeData()
	{}

	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	@Override
	public String getName()
	{
		return name;
	}
	@Override
	public void setName(String name)
	{
		this.name = name;
	}
	private String description;
	@Override
	public String getDescription()
	{
		return description;
	}
	@Override
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private Integer removedBy;
	@Override
	public Integer getRemovedBy()
	{
		return removedBy;
	}
	@Override
	public void setRemovedBy(Integer removedBy)
	{
		this.removedBy = removedBy;
	}
	// -------------------------------------------
	/*
		From the SystemData interface
		-------------------------------------------
	*/
	private String systemId; 
	@Override
	public String getSystemId()
	{
		return systemId;
	}
	public void setSystemId(String systemId)
	{
		this.systemId = systemId;
	}
	// -------------------------------------------

	private Set<BioPlateEventData> events;
	/**
		This is the inverse end.
		@see BioPlateEventData#getEventType()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`bioplateeventtype_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.BioPlateEventData"
	*/
	Set<BioPlateEventData> getEvents()
	{
		return events;
	}
	void setEvents(Set<BioPlateEventData> events)
	{
		this.events = events;
	}

}
