/**
	$Id: AbstractQueryCache.java 7381 2017-05-22 12:35:27Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import java.util.HashMap;
import java.util.Map;

import org.hibernate.query.Query;

/**
	Abstract base class used to cache queries for a session.

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2017-05-22 14:35:27 +0200 (må, 22 maj 2017) $
*/
public abstract class AbstractQueryCache
{

	private final Map<String, Query<?>> cache;
	
	protected AbstractQueryCache()
	{
		this.cache = new HashMap<String, Query<?>>();
	}
	
	/**
		Get a cached query.
		@param key The key (usually the HQL/SQL query string or a named query)
		@return A cached query or null if the query is not cached
	*/
	protected Query<?> getCachedQuery(String key)
	{
		return cache.get(key);
	}
	
	/**
		@since 3.11
	*/
	@SuppressWarnings("unchecked")
	protected <T> Query<T> getCachedQuery(String key, Class<T> resultType)
	{
		return (Query<T>)cache.get(key);
	}
	
	/**
		Cache a query.
		@param key The key (usually the HQL/SQL query string or a named query)
		@param query The query to cache
	*/
	protected void cacheQuery(String key, Query<?> query)
	{
		cache.put(key, query);
	}
	
	
	
}
