/**
	$Id: QueryWrapper.java 7381 2017-05-22 12:35:27Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import java.lang.reflect.Field;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.persistence.FlushModeType;
import javax.persistence.Parameter;
import javax.persistence.TemporalType;

import org.hibernate.CacheMode;
import org.hibernate.FlushMode;
import org.hibernate.LockMode;
import org.hibernate.LockOptions;
import org.hibernate.MappingException;
import org.hibernate.query.NativeQuery;
import org.hibernate.query.QueryParameter;
import org.hibernate.query.internal.AbstractProducedQuery;
import org.hibernate.query.internal.QueryParameterBindingsImpl;
import org.hibernate.query.spi.QueryParameterBinding;
import org.hibernate.engine.query.spi.sql.NativeSQLQueryReturn;
import org.hibernate.type.Type;

/**
	A simple wrapper around hibernate native (SQL) queries which simply forwards 
	all calls to a parent query.
	
	@author nicklas
	@since 3.12.1
*/
public class NativeQueryWrapper<R>
	extends QueryWrapper<R>
	implements NativeQuery<R>
{

	protected final NativeQuery<R> query;
	
	public NativeQueryWrapper(NativeQuery<R> query)
	{
		super(query);
		this.query = query;
	}

	/**
		Get the underlying parent query from Hibernate.
	*/
	public NativeQuery<R> getParentQuery()
	{
		return query;
	}
	
	@Override
	@Deprecated
	public boolean isCallable() 
	{
		return query.isCallable();
	}

	@Override
	@Deprecated
	public List<NativeSQLQueryReturn> getQueryReturns() 
	{
		return query.getQueryReturns();
	}

	@Override
	@Deprecated
	public NativeQuery<R> addScalar(String columnAlias) 
	{
		query.addScalar(columnAlias);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> addScalar(String columnAlias, Type type) 
	{
		query.addScalar(columnAlias, type);
		return this;
	}

	@Override
	@Deprecated
	public RootReturn addRoot(String tableAlias, String entityName) 
	{
		return query.addRoot(tableAlias, entityName);
	}

	@SuppressWarnings("rawtypes")
	@Override
	@Deprecated
	public RootReturn addRoot(String tableAlias, Class entityType) 
	{
		return query.addRoot(tableAlias, entityType);
	}

	@Override
	@Deprecated
	public NativeQuery<R> addEntity(String entityName) 
	{
		query.addEntity(entityName);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> addEntity(String tableAlias, String entityName) 
	{
		query.addEntity(tableAlias, entityName);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> addEntity(String tableAlias, String entityName, LockMode lockMode) 
	{
		query.addEntity(tableAlias, entityName, lockMode);
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	@Deprecated
	public NativeQuery<R> addEntity(Class entityType) 
	{
		query.addEntity(entityType);
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	@Deprecated
	public NativeQuery<R> addEntity(String tableAlias, Class entityType) 
	{
		query.addEntity(tableAlias, entityType);
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	@Deprecated
	public NativeQuery<R> addEntity(String tableAlias, Class entityClass, LockMode lockMode) 
	{
		query.addEntity(tableAlias, entityClass, lockMode);
		return this;
	}

	@Override
	@Deprecated
	public FetchReturn addFetch(String tableAlias, String ownerTableAlias, String joinPropertyName) 
	{
		return query.addFetch(tableAlias, ownerTableAlias, joinPropertyName);
	}

	@Override
	@Deprecated
	public NativeQuery<R> addJoin(String tableAlias, String path) 
	{
		query.addJoin(tableAlias, path);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> addJoin(String tableAlias, String ownerTableAlias, String joinPropertyName) 
	{
		query.addJoin(tableAlias, ownerTableAlias, joinPropertyName);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> addJoin(String tableAlias, String path, LockMode lockMode) 
	{
		query.addJoin(tableAlias, path, lockMode);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setHibernateFlushMode(FlushMode flushMode) 
	{
		query.setHibernateFlushMode(flushMode);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setFlushMode(FlushModeType flushMode) 
	{
		query.setFlushMode(flushMode);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setCacheMode(CacheMode cacheMode) 
	{
		query.setCacheMode(cacheMode);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setCacheable(boolean cacheable) 
	{
		query.setCacheable(cacheable);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setCacheRegion(String cacheRegion) 
	{
		query.setCacheRegion(cacheRegion);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setTimeout(int timeout) 
	{
		query.setTimeout(timeout);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setFetchSize(int fetchSize) 
	{
		query.setFetchSize(fetchSize);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setReadOnly(boolean readOnly) 
	{
		return query.setReadOnly(readOnly);
	}

	@Override
	@Deprecated
	public NativeQuery<R> setLockOptions(LockOptions lockOptions) 
	{
		query.setLockOptions(lockOptions);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setLockMode(String alias, LockMode lockMode) 
	{
		query.setLockMode(alias, lockMode);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> setComment(String comment) 
	{
		query.setComment(comment);
		return this;
	}

	@Override
	@Deprecated
	public NativeQuery<R> addQueryHint(String hint)
	{
		query.addQueryHint(hint);
		return this;
	}

	@Override
	public Collection<String> getSynchronizedQuerySpaces() 
	{
		return query.getSynchronizedQuerySpaces();
	}

	@Override
	@Deprecated
	public NativeQuery<R> setFlushMode(FlushMode flushMode)
	{
		query.setFlushMode(flushMode);
		return this;
	}

	@Override
	public NativeQuery<R> setResultSetMapping(String name) 
	{
		query.setResultSetMapping(name);
		return this;
	}

	@Override
	public <P> NativeQuery<R> setParameter(QueryParameter<P> parameter, P val) 
	{
		query.setParameter(parameter, val);
		return this;
	}

	@Override
	public <P> NativeQuery<R> setParameter(Parameter<P> param, P value) 
	{
		query.setParameter(param, value);
		return this;
	}

	@Override
	public <P> NativeQuery<R> setParameter(QueryParameter<P> parameter, P val, Type type) 
	{
		query.setParameter(parameter, val, type);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(String name, Object val, Type type) 
	{
		query.setParameter(name, val, type);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(int position, Object val, Type type) 
	{
		query.setParameter(position, val, type);
		return this;
	}

	@Override
	public <P> NativeQuery<R> setParameter(QueryParameter<P> parameter, P val, TemporalType temporalType) 
	{
		query.setParameter(parameter, val, temporalType);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(String name, Object val, TemporalType temporalType) 
	{
		query.setParameter(name, val, temporalType);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(int position, Object val, TemporalType temporalType) 
	{
		query.setParameter(position, val, temporalType);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(Parameter<Calendar> param, Calendar value, TemporalType temporalType) 
	{
		query.setParameter(param, value, temporalType);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(Parameter<Date> param, Date value, TemporalType temporalType) 
	{
		query.setParameter(param, value, temporalType);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(String name, Object value)
	{
		query.setParameter(name, value);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(String name, Calendar value, TemporalType temporalType) 
	{
		query.setParameter(name, value, temporalType);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(String name, Date value, TemporalType temporalType) 
	{
		query.setParameter(name, value, temporalType);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(int position, Object value) 
	{
		query.setParameter(position, value);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(int position, Calendar value, TemporalType temporalType) 
	{
		query.setParameter(position, value, temporalType);
		return this;
	}

	@Override
	public NativeQuery<R> setParameter(int position, Date value, TemporalType temporalType) 
	{
		query.setParameter(position, value, temporalType);
		return this;
	}

	@Override
	public <P> NativeQuery<R> setParameterList(QueryParameter<P> parameter, Collection<P> values) 
	{
		query.setParameterList(parameter, values);
		clearOldParameterListParameters(parameter.getName());
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public NativeQuery<R> setParameterList(String name, Collection values) 
	{
		query.setParameterList(name, values);
		clearOldParameterListParameters(name);
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public NativeQuery<R> setParameterList(String name, Collection values, Type type) 
	{
		query.setParameterList(name, values, type);
		clearOldParameterListParameters(name);
		return this;
	}

	@Override
	public NativeQuery<R> setParameterList(String name, Object[] values, Type type) 
	{
		query.setParameterList(name, values, type);
		clearOldParameterListParameters(name);
		return this;
	}

	@Override
	public NativeQuery<R> setParameterList(String name, Object[] values) 
	{
		query.setParameterList(name, values);
		clearOldParameterListParameters(name);
		return this;
	}

	@Override
	public NativeQuery<R> setProperties(Object bean) 
	{
		query.setProperties(bean);
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public NativeQuery<R> setProperties(Map bean) 
	{
		query.setProperties(bean);
		return this;
	}

	@Override
	public NativeQuery<R> addSynchronizedQuerySpace(String querySpace) 
	{
		query.addSynchronizedQuerySpace(querySpace);
		return this;
	}

	@Override
	public NativeQuery<R> addSynchronizedEntityName(String entityName)
		throws MappingException 
	{
		query.addSynchronizedEntityName(entityName);
		return this;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public NativeQuery<R> addSynchronizedEntityClass(Class entityClass) 
		throws MappingException 
	{
		query.addSynchronizedEntityClass(entityClass);
		return this;
	}
	
	
	// Reference to the "QueryParameterBindingsImpl.parameterBindingMap" variable
	private Field parameterBindingMap;
	
	/**
		We need to remove "synthetic" parameters from the "parameterBindingMap".
		The parameters are created by Hibernate when a list-type parameter is 
		specified on a query: name_0, name_1, etc. The problem is that the 
		"synthetic" parameters are never cleared and if the same query is re-used
		with a shorter list, Hibernate will try to set the non-existing parameters
		which causes an exception. See http://base.thep.lu.se/ticket/2110
		UPDATE: We also need to check for copied parameter (name) in case the list 
		contained only a single element. See http://base.thep.lu.se/ticket/2113
	*/
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void clearOldParameterListParameters(String parameterName)
	{
		try
		{
			if (parameterBindingMap == null)
			{
				parameterBindingMap = QueryParameterBindingsImpl.class.getDeclaredField("parameterBindingMap");
				parameterBindingMap.setAccessible(true);
			}
			
			String prefixOfParametersToRemove = parameterName + "_";
			Map<QueryParameter<?>, QueryParameterBinding<?>> m = (Map)parameterBindingMap.get(((AbstractProducedQuery)query).getQueryParameterBindings());
			Iterator<QueryParameter<?>> i = m.keySet().iterator();
			while (i.hasNext())
			{
				String name = i.next().getName();
				if (name.startsWith(prefixOfParametersToRemove) || name.equals(parameterName)) 
				{
					i.remove();
				}
			}
		}
		catch (Exception ex)
		{
			throw new RuntimeException(ex);
		}
	}
}
