/**
	$Id: SimpleErrorHandler.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.error;

/**
 	A simple error handler that either ignores all errors or
 	re-throws them.
 	
	@author nicklas
	@version 2.0
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
*/
public class SimpleErrorHandler
	implements ErrorHandler
{

	private final boolean ignoreAllErrors;
	
	/**
		Create a new simple error handler.
		@param ignoreAllErrors TRUE to ignore all errors, FALSE to re-throw them
	*/
	public SimpleErrorHandler(boolean ignoreAllErrors)
	{
		this.ignoreAllErrors = ignoreAllErrors;
	}
	
	/*
		From the ErrorHandler interface
		-------------------------------------------
	*/
	/**
		Return TRUE if all errors should be ignored, otherwise re-throw the
		throwable.
	*/
	@Override
	public boolean handleError(Throwable t) 
		throws Throwable
	{
		if (ignoreAllErrors) return true;
		throw t;
	}
	// -------------------------------------------

}
