/**
	$Id:ExtensionBean.java 4187 2008-03-20 11:15:25Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

import net.sf.basedb.core.plugin.About;

/**
	A bean-like implementation of the {@link Extension} interface.
	This implementation adds setter methods for simple initialisation
	of the bean properties.

	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public class ExtensionBean<A extends Action>
	implements Extension<A>
{

	private String id;
	private String extensionPoint;
	private ActionFactory<? extends A> actionFactory;
	private RendererFactory<? super A> rendererFactory;
	private About about;
	private float index;
	
	/**
		Create a new empty extension bean. Use the setter
		methods to set the properties before registering 
		the extension with {@link Registry#registerExtension(Extension)}.
	*/
	public ExtensionBean()
	{}
	
	/**
		Create a new fully initialised extension bean. See the 
		{@link Extension} interface for a description of the parameters.
	*/
	public ExtensionBean(
		String id, 
		String extensionPoint, 
		float position,
		About about,
		ActionFactory<? extends A> actionFactory, 
		RendererFactory<? super A> rendererFactory)
	{
		this.id = id;
		this.extensionPoint = extensionPoint;
		this.index = position;
		this.actionFactory = actionFactory;
		this.about = about;
		this.rendererFactory = rendererFactory;
	}
	
	@Override
	public String toString()
	{
		return "ExtensionBean[id="+ id +"]";
	}

	@Override
	public String getId()
	{
		return id;
	}
	public void setId(String id)
	{
		this.id = id;
	}
	
	@Override
	public String getExtends()
	{
		return extensionPoint;
	}
	public void setExtends(String extensionPoint)
	{
		this.extensionPoint = extensionPoint;
	}
	
	@Override
	public float getIndex()
	{
		return index;
	}
	public void setIndex(float index)
	{
		this.index = index;
	}
	
	@Override
	public ActionFactory<? extends A> getActionFactory()
	{
		return actionFactory;
	}
	public void setActionFactory(ActionFactory<? extends A> actionFactory)
	{
		this.actionFactory = actionFactory;
	}
	
	@Override
	public RendererFactory<? super A> getRendererFactory()
	{
		return rendererFactory;
	}
	public void setRendererFactory(RendererFactory<? super A> rendererFactory)
	{
		this.rendererFactory = rendererFactory;
	}

	@Override
	public About getAbout()
	{
		return about;
	}
	public void setAbout(About about)
	{
		this.about = about;
	}

}
