/**
	$Id: ExtraFloatParser.java 5093 2009-09-10 12:36:54Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.importer.spotdata;

import net.sf.basedb.util.Values;
import net.sf.basedb.util.parser.FlatFileParser;

/**
	Parser that is responsible for extracting extra float values
	from the 'spot' section in a BASEfile. Extra value parsers
	are created in the first pass and used in the second pass.

	@author Nicklas
	@version 2.14
	@base.modified $Date: 2009-09-10 14:36:54 +0200 (to, 10 sep 2009) $
*/
public class ExtraFloatParser
{

	private final String id;
	private final int valueIndex;
	
	/**
		Creates a new extra value parser.
		@param id The external id of the extra value type
		@param valueIndex The index of the extra value column
			relative the first column of assay data
	*/
	public ExtraFloatParser(String id, int valueIndex)
	{
		this.id = id;
		this.valueIndex = valueIndex;
	}
	
	/**
		Get the external id of the extra value type.
	*/
	public String getId()
	{
		return id;
	}
	
	/**
		Get the extra value from the data. If the section contains data for 
		multiple bioassays (eg. a matrix BASEfile) this method is called multiple 
		times (with different 'firstIndex' value) for each row.
		 
		@param data The current data line
		@param firstIndex The index of the first data column that contains data
			for the current bioassay
		@return The extracted float value, or Float.NaN if the
			value is invalid (results in not inserting it)
	*/
	public float getValue(FlatFileParser.Data data, int firstIndex)
	{
		return Values.getFloat(data.get(valueIndex + firstIndex), Float.NaN);
	}
	
}
