/**
	$Id: FileSetMemberNameGenerator.java 4740 2009-02-05 13:39:25Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.node;

import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.util.overview.Node;

/**
	Since a FileSetMember isn't Nameable we construct the name
	by combining the name of the file type and the name of the 
	file.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2009-02-05 14:39:25 +0100 (to, 05 feb 2009) $
*/
public class FileSetMemberNameGenerator
	implements NodeNameGenerator<FileSetMember>
{
	public FileSetMemberNameGenerator()
	{}

	/*
		From the NodeNameGenerator interface
		------------------------------------
	*/
	@Override
	public String getNodeName(FileSetMember item, Node parentNode)
	{
		return "member." + item.getId();
	}

	@Override
	public String getNodeTitle(FileSetMember item, Node parentNode)
	{
		String prefix = "Data file";
		String name = "denied";
		try
		{
			prefix = item.getDataFileType().getName();
		}
		catch (PermissionDeniedException ex)
		{}
		try
		{
			name = item.getFile().getName();
		}
		catch (PermissionDeniedException ex)
		{}
		return prefix + ": " + name;
	}
	
	@Override
	public String getDeniedNodeName(Node parentNode)
	{
		return "member";
	}

	@Override
	public String getDeniedNodeTitle(Node parentNode)
	{
		return "Data file: denied";
	}

	@Override
	public String getMissingNodeName(Node parentNode)
	{
		return "member";
	}

	@Override
	public String getMissingNodeTitle(Node parentNode)
	{
		return "Data file: missing";
	}
	
	// ------------------------------------------------
	
}