/*
	$Id: TestJep.java 5320 2010-04-21 09:48:29Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.IntensityTransform;
import net.sf.basedb.util.ma.MACalculator;

public class TestMACalculator
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing MA calculators");

		test_calculate(1, 1, 0, 0);
		test_calculate(2, 1, 1, 0.15051499783199059760686944736225);
		test_calculate(1, 2, -1, 0.15051499783199059760686944736225);
		
		double ch1 = Math.ceil(Math.random() * 100+10);
		double ch2 = Math.ceil(Math.random() * 100+10);
		double M = Math.log(ch1/ch2) / Math.log(2);
		double A = Math.log10(ch1*ch2) / 2;
		test_calculate(ch1, ch2, M, A);
		test_calculate(ch2, ch1, -M, A);
		
		test_correct(1, 1, 0, 1, 1);
		test_correct(2, 2, 1, 1, 4);
		double factor = Math.random()-0.5;
		
		test_correct(ch1, ch2, factor, ch1 / Math.pow(2, factor) , ch2 * Math.pow(2, factor));
		
		write("++Testing MA calculators "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static double log2(double v)
	{
		return Math.log(v) / Math.log(2);
	}
	
	static void test_calculate(double ch1, double ch2, double M, double A)
	{
		String calcSummary = "MA calculation: ["+ch1+","+ch2+"]->["+M+","+A+"]";

		for (IntensityTransform tf : IntensityTransform.values())
		{
			MACalculator mac = tf.getMACalculator();
			double[] MA = mac.MA(tf.transform(ch1), tf.transform(ch2));
			
			if (!compareOk(MA[0], M) || !compareOk(MA[1], A))
			{
				write("--" + calcSummary +": FAILED "+mac.getClass().getSimpleName()+"["+MA[0]+","+MA[1]+"]");
				ok = false;
			}
		}

		if (ok)
		{
			write("--" + calcSummary + ": OK");
		}
	}
	
	static void test_correct(double ch1, double ch2, double factor, double ch1New, double ch2New)
	{
		String calcSummary = "Correction calculation: ["+ch1+","+ch2+"]("+factor+")->["+ch1New+","+ch2New+"]";

		for (IntensityTransform tf : IntensityTransform.values())
		{
			MACalculator mac = tf.getMACalculator();
			double[] c = mac.correct(tf.transform(ch1), tf.transform(ch2), factor);
			
			if (!compareOk(c[0], tf.transform(ch1New)) || !compareOk(c[1], tf.transform(ch2New)))
			{
				write("--" + calcSummary +": FAILED "+mac.getClass().getSimpleName()+"["+c[0]+","+c[1]+"]");
				ok = false;
			}
		}

		if (ok)
		{
			write("--" + calcSummary + ": OK");
		}
	}

	
	static boolean compareOk(double v1, double v2)
	{
		return Math.abs(v1-v2) < 0.0000001;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	

}

