/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var SelectPlugin = function()
{
	var select = {};

	/**
		Initialize the page.
	*/
	select.initPage = function()
	{
		// Buttons
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnNext', select.next);
		
		Events.addEventHandler('plugins', 'change', select.pluginOnChange);
		select.initPluginList();
		select.pluginOnChange();
		
		Buttons.addClickHandler('pluginMessages', select.togglePluginMessages);
	}
	
	
	select.initPluginList = function()
	{
		var list = Doc.element('plugins');
		var plugins = Data.json('page-data', 'plugins');
		// Add 'auto-detect' option
		// (will be removed again if no plug-in supports auto-detection)
		var option = new Option('- auto detect (all) -', '-1');
		list[list.length] = option;
		option.className = 'selectoptionautodetect';
		
		var numAutodetecting = 0;
		for (var i = 0; i < plugins.length; i++)
		{
			var plugin = plugins[i];
			var option = new Option(plugin.name, plugin.id);
			if (plugin.autoDetect) 
			{
				option.text += ' ×';
				numAutodetecting++;
			}
			option.plugin = plugin;
			option.className = 'selectoptionheader';
			option.disabled = plugin.requiresConfig;
			list[list.length] = option;
			
			if (plugin.supportsConfig && plugin.configurations.length > 0)
			{
				if (plugin.autoDetect)
				{
					var option = new Option(' ├ auto detect (' + plugin.name + ') -', '-1');
					option.className = 'selectoptionautodetect';
					option.plugin = plugin;
					list[list.length] = option;
				}
				for (var j = 0; j < plugin.configurations.length; j++)
				{
					var conf = plugin.configurations[j];
					var prefix = j == plugin.configurations.length-1 ? ' └ ' : ' ├ '
					var option = new Option(prefix+conf.name, conf.id);
					option.plugin = plugin;
					option.configuration = conf;
					list[list.length] = option;
				}
			}
		}
		if (numAutodetecting == 0)
		{
			list[0] = null;
		}
	}
	
	select.pluginOnChange = function()
	{
		var frm = document.forms['plugin'];
		var selected = frm.plugins[frm.plugins.selectedIndex];
		var plugin = selected.plugin;
		var config = selected.configuration;
		var selectedValue = selected.value;
		
		var descriptions = Doc.element('descriptions');
		descriptions.innerHTML = '<i>select plug-in above</i>';
		if (plugin)
		{
			descriptions.innerHTML = plugin.description;
			frm.plugindefinition_id.value = plugin.id;
			frm.pluginconfiguration_id.value = selectedValue == -1 ? -1 : 0;
		}
		else
		{
			frm.plugindefinition_id.value = selectedValue;
		}
		if (config)
		{
			descriptions.innerHTML += '<p>'+config.description;
			frm.pluginconfiguration_id.value = config.id;
		}
	}
	
	select.togglePluginMessages = function()
	{
		Doc.showHide('pluginMessages.content');
	}
	
	select.next = function()
	{
		var frm = document.forms['plugin'];
		var anotherWizardIsOpen = Data.int('page-data', 'another-wizard');
		if (anotherWizardIsOpen)
		{
			if (!confirm('Are you sure? Unsaved settings in the other wizard will be lost.'))
			{
				return;
			}
			frm.warnIfOpen.value = '0';
		}
		frm.submit();
	}
	
	return select;
}();

Doc.onLoad(SelectPlugin.initPage);