/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Formulas = function()
{
	var formulas = {};
	
	/**
		Initialize the page.
	*/
	formulas.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', formulas.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabValidator('settings.info', formulas.validateFormula);
			
			// Channels
			Events.addEventHandler('channels', 'keypress', Events.integerOnly);

			// Color values
			Events.addEventHandler('useColors', 'click', formulas.useColorsOnClick);
			Events.addEventHandler('min_value', 'keypress', Events.numberOnly);
			Events.addEventHandler('mid_value', 'keypress', Events.numberOnly);
			Events.addEventHandler('max_value', 'keypress', Events.numberOnly);
			
			// Raw data type
			Events.addEventHandler('rawdatatype', 'change', formulas.rawDataTypeOnChange);
			
			// Formula type
			Events.addEventHandler('formulaType', 'change', formulas.formulaTypeOnChange);
			formulas.formulaTypeOnChange();
			
			// Expressions
			Events.addEventHandler('expressions', 'change', formulas.expressionsOnClick);
			Events.addEventHandler('expression', 'keyup', formulas.expressionOnBlur);
			Events.addEventHandler('expression', 'change', formulas.expressionOnChange);
			Buttons.addClickHandler('btnRemoveExpression', formulas.removeOnClick);
			Buttons.addClickHandler('btnAddExpression', formulas.addOnClick);
			Buttons.addClickHandler('btnExpressionBuilder', formulas.openExpressionBuilder);
			
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'FORMULA', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'FORMULA'};
			var tableAttributes = {'table-id': 'formulas'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	formulas.validateFormula = function()
	{
		var frm = document.forms['formula'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		
		var type = frm.type.value;
		var numFormulas = frm.expressions.length;
		var numChannels = frm.channels.value;
		if (type == 'INTENSITY_EXPRESSION' || type == 'INTENSITY_TRANSFORMATION')
		{
			// One formula per channel required
			if (numFormulas != numChannels)
			{
				Forms.showNotification(frm.expressions, 'You must enter one formula per channel for type = ' + frm.type[frm.type.selectedIndex].text, null, 'pointer-left');
				return false;
			}
		}
		else
		{
			// Only one formula can be specified
			if (numFormulas != 1)
			{
				Forms.showNotification(frm.expressions, 'You must enter exactly one formula for type = ' + frm.type[frm.type.selectedIndex].text, null, 'pointer-left');
				return false;
			}
		}
		
		if (frm.use_colors.checked && !frm.use_colors.disabled)
		{
			var minValue = parseFloat(frm.min_value.value);
			var midValue = parseFloat(frm.mid_value.value);
			var maxValue = parseFloat(frm.max_value.value);
			if (isNaN(minValue))
			{
				Forms.showNotification(frm.min_value, 'No min value has been specified');
				return false;
			}
			if (isNaN(midValue))
			{
				Forms.showNotification(frm.mid_value, 'No mid value has been specified');
				return false;
			}
			if (isNaN(maxValue))
			{
				Forms.showNotification(frm.max_value, 'No max value has been specified');
				return false;
			}
			if (minValue > midValue)
			{
				Forms.showNotification(frm.mid_value, 'The mid value must be greater than the min value');
				return false;
			}
			if (midValue > maxValue)
			{
				Forms.showNotification(frm.max_value, 'The max value must be greater than the mid value');
				return false;
			}
		}
		return true;
	}

	formulas.save = function()
	{
		var frm = document.forms['formula'];
		if (TabControl.validateActiveTab('settings'))
		{
			for (var i = 0; i < frm.expressions.length; i++)
			{
				Forms.addHidden(frm, 'formulas', frm.expressions[i].text);
			}
			frm.channels.disabled = false; // Otherwise the channels info, isn't sent
			frm.submit();
		}
	}
	
	formulas.useColorsOnClick = function()
	{
		var frm = document.forms['formula'];
		var useColors = frm.use_colors.checked && !frm.use_colors.disabled;
		frm.min_value.disabled = !useColors;
		frm.mid_value.disabled = !useColors;
		frm.max_value.disabled = !useColors;
		frm.logarithmic.disabled = !useColors;
		if (useColors)
		{
			Doc.addClass(frm.min_value, 'required');
			Doc.addClass(frm.mid_value, 'required');
			Doc.addClass(frm.max_value, 'required');
		}
		else
		{
			Doc.removeClass(frm.min_value, 'required');
			Doc.removeClass(frm.mid_value, 'required');
			Doc.removeClass(frm.max_value, 'required');
		}
	}

	formulas.rawDataTypeOnChange = function()
	{
		var frm = document.forms['formula'];
		var rawDataType = frm.rawdatatype[frm.rawdatatype.selectedIndex];
		if (rawDataType.value != '')
		{
			frm.channels.value = Data.get(rawDataType, 'channels');
			frm.channels.disabled = true;
			Doc.addClass(frm.channels, 'disabled');
			Doc.removeClass(frm.channels, 'required');
		}
		else
		{
			frm.channels.disabled = false;
			Doc.removeClass(frm.channels, 'disabled');
			Doc.addClass(frm.channels, 'required');
		}
	}

	formulas.formulaTypeOnChange = function()
	{
		var frm = document.forms['formula'];
		var formulaType = frm.type[frm.type.selectedIndex];
		
		// Enable/disable colors
		frm.use_colors.disabled = !Data.int(formulaType, 'supports-colors');
		formulas.useColorsOnClick();
		
		// Enable/disable average method
		var supportsAvg = Data.int(formulaType, 'supports-average');
		frm.averageMethod.disabled = !supportsAvg;
		if (!supportsAvg) frm.averageMethod.selectedIndex = 0;
		
		// Value type
		var valueType = Data.get(formulaType, 'value-type');
		if (valueType)
		{
			Forms.selectListOption(frm.valueType, valueType);
			frm.valueType.disabled = true;
		}
		else
		{
			frm.valueType.disabled = false;
		}
	}

	formulas.expressionsOnClick = function()
	{
		var frm = document.forms['formula'];
		var selectedExpressionIndex = frm.expressions.selectedIndex;
		if (selectedExpressionIndex >= 0)
		{
			frm.expression.value = frm.expressions[selectedExpressionIndex].text;
			Data.set(frm.expression, 'linked-index', selectedExpressionIndex);
		}
	}
	
	formulas.expressionOnBlur = function()
	{
		var frm = document.forms['formula'];
		var selectedExpressionIndex = Data.int(frm.expression, 'linked-index', -1);
		if (selectedExpressionIndex >= 0)
		{
			frm.expressions[selectedExpressionIndex].text = frm.expression.value;
		}
	}
	
	formulas.removeOnClick = function()
	{
		var frm = document.forms['formula'];
		for (var i = frm.expressions.length-1; i >= 0; i--)
		{
			if (frm.expressions[i].selected) frm.expressions[i] = null;
		}
		frm.expression.value = '';
		Data.set(frm.expression, 'linked-index', -1);
	}
	
	formulas.addOnClick = function()
	{
		var frm = document.forms['formula'];
		if (frm.expression.value != '')
		{
			frm.expressions[frm.expressions.length] = new Option(frm.expression.value);
			frm.expressions.selectedIndex = frm.expressions.length - 1;
			Data.set(frm.expression, 'linked-index', frm.expressions.selectedIndex);
		}
	}

	formulas.openExpressionBuilder = function()
	{
		var frm = document.forms['formula'];
		var rdt = frm.rawdatatype.value;
		var formulaType = frm.type.value;
		var channels = frm.channels.value;
		var restrictions = formulaType == 'COLUMN_RESTRICTION';
		Dialogs.openExpressionBuilder('expression', 'Expression', formulaType, rdt, channels)
	}


	formulas.expressionOnChange = function(event)
	{
		var frm = document.forms['formula'];
		if (frm.expressions.length == 0) 
		{
			formulas.addOnClick();
		}
		else
		{
			formulas.expressionOnBlur();
		}
	}

	return formulas;
}();

Doc.onLoad(Formulas.initPage);
